import requests
import urllib
import logging

from datetime import datetime, timedelta
from uuid import uuid4
from django import forms
from django.conf import settings
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from smartmin.views import SmartFormView

from temba.channels.views import ClaimViewMixin
from temba.channels.models import Channel


DEFAULT_STATE_KEY = 'wassup_auth_state'
DEFAULT_AUTHORIZATION_KEY = 'wassup_authorizations'
DEFAULT_AUTH_URL = 'https://wassup.p16n.org'
DEFAULT_SCOPES = " ".join([
    "numbers:read",
    "messages:read",
    "messages:write",
    "websocket:read",
    "profile:read",
])

logger = logging.getLogger(__name__)


class NumberForm(ClaimViewMixin.Form):

    number = forms.ChoiceField(
        required=True,
        help_text=_('Which number would you like to connect?'))


class GroupForm(ClaimViewMixin.Form):

    group = forms.ChoiceField(
        required=True,
        help_text=_('Which group would you like to connect?'))


class WhatsAppClaimView(ClaimViewMixin, SmartFormView):

    def pre_process(self, *args, **kwargs):
        response = super(WhatsAppClaimView, self).pre_process(
            *args, **kwargs)

        code = self.request.GET.get('code')
        state = self.request.GET.get('state')
        session_state = self.request.session.get(DEFAULT_STATE_KEY)
        if all([code, state]) and state == session_state:
            self.set_session_authorization(
                self.get_authorization(code))
            del self.request.session[DEFAULT_STATE_KEY]
            return redirect(
                reverse('channels.claim_%s' % (
                    self.channel_type.slug,)))

        return response

    def set_session_authorization(self, authorization):
        self.request.session[DEFAULT_AUTHORIZATION_KEY] = authorization

    def get_session_authorization(self):
        return self.request.session.get(
            DEFAULT_AUTHORIZATION_KEY, {})

    def clear_session_authorization(self):
        del self.request.session[DEFAULT_AUTHORIZATION_KEY]

    def get_authorization(self, code):
        wassup_url = getattr(
            settings, 'WASSUP_AUTH_URL', DEFAULT_AUTH_URL)
        client_id = getattr(
            settings, 'WASSUP_AUTH_CLIENT_ID', None)
        client_secret = getattr(
            settings, 'WASSUP_AUTH_CLIENT_SECRET', None)

        redirect_uri = self.get_redirect_uri()

        response = requests.post(
            '%s/oauth/token/' % (wassup_url,), {
                "grant_type": "authorization_code",
                "code": code,
                "redirect_uri": redirect_uri,
                "client_id": client_id,
                "client_secret": client_secret,
            })
        response.raise_for_status()
        return response.json()

    def get_redirect_uri(self):
        return self.request.build_absolute_uri(
            reverse('channels.claim_%s' % (self.channel_type.slug,)))

    def get_context_data(self, **kwargs):
        context = super(WhatsAppClaimView, self).get_context_data(
            **kwargs)

        wassup_url = getattr(
            settings, 'WASSUP_AUTH_URL', DEFAULT_AUTH_URL)
        client_id = getattr(
            settings, 'WASSUP_AUTH_CLIENT_ID', None)
        auth_scopes = getattr(
            settings, 'WASSUP_AUTH_SCOPES', DEFAULT_SCOPES
        )

        auth_state = uuid4().hex[:8]
        self.request.session[DEFAULT_STATE_KEY] = auth_state

        auth_url = '%s/oauth/authorize/?%s' % (
            wassup_url,
            urllib.urlencode({
                "client_id": client_id,
                "redirect_uri": self.get_redirect_uri(),
                "scopes": auth_scopes,
                "response_type": "code",
                "state": auth_state,
            }))

        context['whatsapp_auth_url'] = auth_url

        # if we've not authorized yet, remove the form
        authorization = self.get_session_authorization()
        context['authorization'] = authorization
        if authorization.get('access_token') is not None:
            context['show_form'] = True
        else:
            context['show_form'] = False
        return context

    def wassup_url(self):
        return getattr(
            settings, 'WASSUP_API_URL', '%s/api/v1' % (DEFAULT_AUTH_URL,))

    def get_numbers(self, api_token):
        response = requests.get(
            '%s/numbers/' % (self.wassup_url(),),
            headers={
                'Authorization': 'Bearer %s' % (api_token),
                'Accept': 'application/json',
            })
        response.raise_for_status()
        data = response.json()
        return data['results']

    def get_number_choices(self, api_token):
        return [(
            '%(from_addr)s' % number,
            '%(vname)s (%(from_addr)s)' % number,
        ) for number in self.get_numbers(api_token)]

    def get_groups(self, api_token):
        response = requests.get(
            '%s/groups/' % (self.wassup_url(),),
            headers={
                'Authorization': 'Bearer %s' % (api_token),
                'Accept': 'application/json',
            })
        response.raise_for_status()
        data = response.json()
        return data['results']

    def get_group_choices(self, api_token):
        return [(group['uuid'], '%(subject)s for %(number)s' % group)
                for group in self.get_groups(api_token)]


class DirectClaimView(WhatsAppClaimView):

    def get_form_class(self, *args, **kwargs):
        return NumberForm

    # NOTE: this is a SmartMin callback
    def customize_form_field(self, name, field):
        authorization = self.request.session.get(DEFAULT_AUTHORIZATION_KEY)
        if authorization and name == 'number':
            field.choices = self.get_number_choices(
                authorization['access_token'])
        return field

    def form_valid(self, form):
        org = self.request.user.get_org()
        number = form.cleaned_data['number']
        authorization = self.get_session_authorization()

        config = {
            'authorization': authorization,
            'expires_at': (
                datetime.now() + timedelta(
                    seconds=authorization['expires_in'])).isoformat(),
            'number': number,
        }

        self.object = Channel.create(
            org, self.request.user, None, self.channel_type,
            name='Direct Messages to %s' % (number,),
            address=number, config=config,
            secret=Channel.generate_secret())

        self.clear_session_authorization
        return super(WhatsAppClaimView, self).form_valid(form)


class GroupClaimView(WhatsAppClaimView):

    def get_form_class(self, *args, **kwargs):
        return GroupForm

    # NOTE: this is a SmartMin callback
    def customize_form_field(self, name, field):
        authorization = self.request.session.get(DEFAULT_AUTHORIZATION_KEY)
        if authorization and name == 'group':
            field.choices = self.get_group_choices(
                authorization['access_token'])
        return field

    def form_valid(self, form):
        org = self.request.user.get_org()
        authorization = self.get_session_authorization()
        access_token = authorization['access_token']

        group_uuid = form.cleaned_data['group']
        [group] = [group
                   for group in self.get_groups(access_token)
                   if group['uuid'] == group_uuid]

        config = {
            'authorization': authorization,
            'expires_at': (
                (datetime.now() + timedelta(
                    seconds=authorization['expires_in']))).isoformat(),
            'group': group,
        }

        self.object = Channel.create(
            org, self.request.user, None, self.channel_type,
            name='group messages to %(subject)s on %(number)s' % group,
            address=group['number'], config=config,
            secret=Channel.generate_secret())

        self.clear_session_authorization()
        return super(WhatsAppClaimView, self).form_valid(form)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('VERSION') as fp:
    version = fp.read().strip()

setup(
    name='warapidpro',
    version=version,
    description="warapidpro",
    long_description=readme,
    author="Simon de Haan",
    author_email='simon@praekelt.org',
    url='https://github.com/praekeltfoundation/wa-rapidpro',
    packages=[
        'warapidpro',
    ],
    package_dir={'warapidpro':
                 'warapidpro'},
    include_package_data=True,
    install_requires=[],
    zip_safe=False,
    keywords='warapidpro',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ]
)



from __future__ import absolute_import

from django.apps import AppConfig
import logging

logger = logging.getLogger(__name__)


class IntegrationConfig(AppConfig):
    name = 'warapidpro'
    # NOTE: This is to maintain backwards compatibility with
    #       existing installations this was extracted from
    label = 'integration'
    verbose_name = "WhatsApp RapidPro integration"

    def ready(self):
        from temba.channels import types
        from .types import WhatsAppDirectType, WhatsAppGroupType
        from .handlers import WhatsAppHandler

        # NOTE: Loading WhatsAppHandler so when RapidPro
        # looks for ChannelHandler implementations it will
        # load this one into the urlpatterns too
        WhatsAppHandler

        types.register_channel_type(WhatsAppDirectType)
        types.register_channel_type(WhatsAppGroupType)

        logger.info('Registered the WhatsApp Channel')



import logging
import requests
import json
import time
import os.path
import six
from io import BytesIO

from temba.channels.models import (
    Channel, ChannelType, TEMBA_HEADERS, SendException)
from temba.msgs.models import WIRED, Msg, Attachment
from temba.contacts.models import TEL_SCHEME
from temba.utils.http import HttpEvent
from django.shortcuts import reverse
from django.conf import settings

from .views import DirectClaimView, GroupClaimView

logger = logging.getLogger(__name__)

WHATSAPP_DIRECT_CHANNEL_TYPE = 'WAD'
WHATSAPP_GROUP_CHANNEL_TYPE = 'WAG'

WHATSAPP_CHANNEL_TYPES = set([
    WHATSAPP_DIRECT_CHANNEL_TYPE,
    WHATSAPP_GROUP_CHANNEL_TYPE,
])


class WhatsAppType(ChannelType):
    category = ChannelType.Category.SOCIAL_MEDIA
    scheme = TEL_SCHEME
    max_length = 4096
    max_tps = None
    attachment_support = True
    free_sending = True

    def channel_url(self, channel):
        return 'https://%s%s' % (
            settings.HOSTNAME,
            reverse('handlers.whatsapp_handler', args=(channel.uuid,)))

    def wassup_url(self):
        return getattr(
            settings, 'WASSUP_API_URL', 'https://wassup.p16n.org/api/v1')

    def add_channel_webhook(self, channel, event):
        headers = self.api_request_headers(channel)
        headers.update({
            'Content-Type': 'application/json',
        })

        response = requests.post(
            '%s/webhooks/' % (self.wassup_url(),),
            json={
                'event': event,
                'url': self.channel_url(channel),
                'number': channel.address,
                'secret': channel.secret,
            },
            headers=headers)
        response.raise_for_status()
        data = response.json()
        return data['id']

    def remove_channel_webhook(self, channel, webhook_id):
        headers = self.api_request_headers(channel)

        response = requests.delete(
            '%s/webhooks/%s/' % (self.wassup_url(), webhook_id,),
            headers=headers)
        response.raise_for_status()

    def remove_channel_webhooks(self, channel):
        for webhook_id in channel.config_json().get('wassup_webhook_ids', []):
            self.remove_channel_webhook(channel, webhook_id)

    def fetch_attachment(self, attachment):
        """
        NOTE:   this isn't using the streaming API that requests
                has because I'm failing to write a test for it
                If memory usage is going through the roof,
                look here.
        """
        category = attachment.content_type.split('/')[0]
        attachment_type = {
            'audio': 'audio_attachment',
            'image': 'image_attachment',
            'video': 'video_attachment',
            'application': 'document_attachment',
        }.get(category)
        if not attachment_type:
            logger.warning(
                'Trying to send unsupported attachment type: %s' % (
                    category,))
            return {}

        response = requests.get(attachment.url, stream=True)
        response.raise_for_status()
        return {
            attachment_type: (
                os.path.basename(attachment.url),
                BytesIO(response.content),
                attachment.content_type),
        }

    def api_request_headers(self, channelish):
        if(isinstance(channelish, Channel)):
            config = channelish.config_json()
        else:
            config = channelish.config

        headers = TEMBA_HEADERS.copy()
        headers.update({
            'Accept': 'application/json',
        })

        if 'api_token' in config:
            headers.update({
                'Authorization': 'Token %s' % (
                    config['api_token'],),
            })
        else:
            authorization = config['authorization']
            headers.update({
                'Authorization': '%s %s' % (
                    authorization['token_type'],
                    authorization['access_token'],
                )
            })
        return headers

    def send_whatsapp(self, channel_struct, msg, payload, attachments=None):
        url = ('%s/messages/' % (self.wassup_url(),))
        headers = self.api_request_headers(channel_struct)
        event = HttpEvent('POST', url, json.dumps(payload))
        start = time.time()

        # Grab the first attachment if it exists
        attachments = Attachment.parse_all(msg.attachments)
        attachment = attachments[0] if attachments else None

        try:
            if attachment:
                files = self.fetch_attachment(attachment)
                data = payload
            else:
                headers.update({
                    'Content-Type': 'application/json'
                })
                data = json.dumps(payload)
                files = {}

            response = requests.post(
                url, data=data, files=files, headers=headers)
            response.raise_for_status()
            event.status_code = response.status_code
            event.response_body = response.text
        except (requests.RequestException,) as e:
            raise SendException(
                'error: %s, request: %r, response: %r' % (
                    six.text_type(e), e.request.body, e.response.content),
                event=event, start=start)

        data = response.json()
        try:
            message_id = data['uuid']
            Channel.success(channel_struct, msg, WIRED, start,
                            event=event, external_id=message_id)
        except (KeyError,) as e:
            raise SendException(
                "Unable to read external message_id: %r" % (e,),
                event=HttpEvent('POST', url,
                                request_body=json.dumps(json.dumps(payload)),
                                response_body=json.dumps(data)),
                start=start)


class WhatsAppDirectType(WhatsAppType):

    claim_blurb = 'Claim your direct WhatsApp Channel'
    claim_view = DirectClaimView
    show_config_page = False
    code = WHATSAPP_DIRECT_CHANNEL_TYPE
    slug = 'wad'
    name = 'WhatsApp Direct'

    def activate(self, channel_struct):
        channel = Channel.objects.get(id=channel_struct.id)
        logger.info('Activating channel %s' % (channel,))
        dm_id = self.add_channel_webhook(
            channel, 'message.direct_inbound')
        status_id = self.add_channel_webhook(
            channel, 'message.direct_outbound.status')
        config = channel.config_json()
        config.update({
            'wassup_webhook_ids': [dm_id, status_id]
        })
        channel.config = json.dumps(config)
        channel.save(update_fields=['config'])

    def deactivate(self, channel_struct):
        channel = Channel.objects.get(id=channel_struct.id)
        logger.info('Deactivating channel %s' % (channel,))
        self.remove_channel_webhooks(channel)

    def activate_trigger(self, trigger):
        logger.info('Activating trigger %s' % (trigger,))

    def deactivate_trigger(self, trigger):
        logger.info('Deactivating trigger %s' % (trigger,))

    def send(self, channel_struct, msg, text):
        logger.info(
            'Sending message %s to channel %s, text: %s' % (
                msg, channel_struct, text))
        self.send_whatsapp(channel_struct, msg, {
            'to_addr': msg.urn_path,
            'number': channel_struct.address,
            'group': '',
            'in_reply_to': (
                Msg.objects.values_list(
                    'external_id', flat=True).filter(
                        pk=msg.response_to_id).first()) or '',
            'content': text,
        })


class WhatsAppGroupType(WhatsAppType):

    claim_blurb = 'Claim your WhatsApp Group Channel'
    claim_view = GroupClaimView
    show_config_page = False
    code = WHATSAPP_GROUP_CHANNEL_TYPE
    slug = 'wag'
    name = 'WhatsApp Group'

    def activate(self, channel_struct):
        channel = Channel.objects.get(id=channel_struct.id)
        logger.info('Activating channel %s' % (channel,))
        dm_id = self.add_channel_webhook(
            channel, 'message.group_inbound')
        status_id = self.add_channel_webhook(
            channel, 'message.group_outbound.status')
        config = channel.config_json()
        config.update({
            'wassup_webhook_ids': [dm_id, status_id]
        })
        channel.config = json.dumps(config)
        channel.save(update_fields=['config'])

    def deactivate(self, channel_struct):
        channel = Channel.objects.get(id=channel_struct.id)
        logger.info('Deactivating channel %s' % (channel,))
        self.remove_channel_webhooks(channel)

    def activate_trigger(self, trigger):
        logger.info('Activating trigger %s' % (trigger,))

    def deactivate_trigger(self, trigger):
        logger.info('Deactivating trigger %s' % (trigger,))

    def send(self, channel_struct, msg, text):
        logger.info(
            'Sending message %s to channel %s, text: %s' % (
                msg, channel_struct, text))
        self.send_whatsapp(channel_struct, msg, {
            'to_addr': msg.urn_path,
            'number': channel_struct.address,
            'group': channel_struct.config.get('group_uuid'),
            'in_reply_to': (
                Msg.objects.values_list(
                    'external_id', flat=True).filter(
                        pk=msg.response_to_id).first()) or '',
            'content': text,
        })
